#region Copyright Syncfusion Inc. 2001 - 2019
//
//  Copyright Syncfusion Inc. 2001 - 2019. All rights reserved.
//
//  Use of this code is subject to the terms of our license.
//  A copy of the current license can be obtained at any time by e-mailing
//  licensing@syncfusion.com. Any infringement will be prosecuted under
//  applicable laws. 
//
#endregion

using System.Diagnostics;
using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.Text.RegularExpressions;
using Syncfusion.Windows.Forms.Edit.Interfaces;
using Syncfusion.Windows.Forms.Edit.Implementation.Config;
using Syncfusion.Windows.Forms.Edit.Implementation.Parser;
using Syncfusion.Windows.Forms.Edit.Enums;
using Syncfusion.Windows.Forms.Edit.Implementation;
using Syncfusion.Windows.Forms;
using Syncfusion.Windows.Forms.Tools.XPMenus;
using Syncfusion.Licensing;
using System.IO;
using Syncfusion.WinForms.Controls;

namespace AdvancedIntellisenseDemo
{
    /// <summary>
    /// Summary description for Form1.
    /// </summary>
    public class Form1 : SfForm
    {
        #region Class Private Members
        /// <summary>
        /// Hashtable with descriptions assigned to the lexem configuration.
        /// </summary>
        private Hashtable m_MethodComments = new Hashtable();
        /// <summary>
        /// Hashtable of the prompts list assigned to the lexem configuration.
        /// </summary>
        private Hashtable m_MethodPrompts = new Hashtable();
        #endregion
        private System.ComponentModel.IContainer components;

        #region Class Controls

        private Syncfusion.Windows.Forms.Edit.EditControl editControl1;

        private System.Windows.Forms.ImageList imageList1;
        #endregion

        #region Class Initialization/Finalization
        /// <summary>
        /// Creates and initiallizes form.
        /// </summary>
        public Form1()
        {
            InitializeComponent();

            this.editControl1.Style = Syncfusion.Windows.Forms.Edit.EditControlStyle.Office2016White;
            // Creates new file using C# language configuration.
            editControl1.NewFile(editControl1.Configurator["C#"]);

            // Creates format for methods coloring.
            ISnippetFormat formatMethod = editControl1.Language.Add("Method", "Text");
            formatMethod.FontColor = Color.HotPink;

            // Creates format for properties coloring.
            ISnippetFormat formatProperty = editControl1.Language.Add("Property", "Text");
            formatProperty.FontColor = Color.Goldenrod;

            // Disables ContextChoiceList dropping after dot in global context.
            ConfigLexem lexemDot_ = GetConfigLexem((IConfigLexem)editControl1.Language, ".");
            lexemDot_.DropContextChoiceList = false;


            // Add configuration for "Property1"
            ConfigLexem this_Property1 = GetConfigLexem((IConfigLexem)editControl1.Language, "var1");
            this_Property1.Type = FormatType.Custom; this_Property1.FormatName = "Property";
            this_Property1.IsComplex = true;

            // Add configuration for "." after "Property1"
            ConfigLexem propertyDot = GetConfigLexem((IConfigLexem)this_Property1, ".");
            propertyDot.Type = FormatType.Operator;
            propertyDot.DropContextChoiceList = true;
            propertyDot.IsComplex = true;
            propertyDot.Priority = 10;

            // Add "Method1" configuration after "this.Property1."
            ConfigLexem property_Method1 = GetConfigLexem((IConfigLexem)propertyDot, "MethodA");
            property_Method1.Type = FormatType.Custom; property_Method1.FormatName = "Method";
            property_Method1.IsComplex = true;

            // Add "Method2" configuration after "this.Property1."
            ConfigLexem property_Method2 = GetConfigLexem((IConfigLexem)propertyDot, "MethodB");
            property_Method2.Type = FormatType.Custom; property_Method2.FormatName = "Method";
            property_Method2.IsComplex = true;

            editControl1.Language.ResetCaches();

            m_MethodComments[this_Property1.ID] = "The only one property\nIt has multiline description";
            m_MethodComments[property_Method1.ID] = "Method 1";
            m_MethodComments[property_Method2.ID] = "Method 2";

            editControl1.Text = @"public class abcd     
{
    public void MethodA() { }
    public void MethodB() { }
}

public class xyz
    {
        public void MethodM() 
        {
            abcd var1 = new abcd();";

            // Resets undo buffer, so the user cannot undo setting of the text.
            editControl1.ResetUndoInfo();
        }

        /// <summary>
        /// Clean up any resources being used.
        /// </summary>
        protected override void Dispose(bool disposing)
        {
            if (disposing)
            {
                if (components != null)
                {
                    components.Dispose();
                }
            }
            base.Dispose(disposing);
        }

        #endregion

        #region Windows Form Designer generated code
        /// <summary>
        /// Required method for Designer support - do not modify
        /// the contents of this method with the code editor.
        /// </summary>
        private void InitializeComponent()
        {
            this.components = new System.ComponentModel.Container();
            System.ComponentModel.ComponentResourceManager resources = new System.ComponentModel.ComponentResourceManager(typeof(Form1));
            Syncfusion.Windows.Forms.Edit.Implementation.Config.Config config1 = new Syncfusion.Windows.Forms.Edit.Implementation.Config.Config();
            this.imageList1 = new System.Windows.Forms.ImageList(this.components);
            this.editControl1 = new Syncfusion.Windows.Forms.Edit.EditControl();

            ((System.ComponentModel.ISupportInitialize)(this.editControl1)).BeginInit();

            this.SuspendLayout();
            // 
            // imageList1
            // 
            this.imageList1.ImageStream = ((System.Windows.Forms.ImageListStreamer)(resources.GetObject("imageList1.ImageStream")));
            this.imageList1.TransparentColor = System.Drawing.Color.Transparent;
            this.imageList1.Images.SetKeyName(0, "");
            this.imageList1.Images.SetKeyName(1, "");
            this.imageList1.Images.SetKeyName(2, "");
            // 
            // editControl1
            // 
            this.editControl1.AllowZoom = false;
            this.editControl1.ChangedLinesMarkingLineColor = System.Drawing.Color.FromArgb(((int)(((byte)(255)))), ((int)(((byte)(238)))), ((int)(((byte)(98)))));
            this.editControl1.CodeSnipptSize = new System.Drawing.Size(100, 100);
            this.editControl1.ColumnGuidesMeasuringFont = new System.Drawing.Font("Segoe UI", 8.25F);
            this.editControl1.Configurator = config1;
            this.editControl1.ContextChoiceBackColor = System.Drawing.Color.FromArgb(((int)(((byte)(255)))), ((int)(((byte)(255)))), ((int)(((byte)(255)))));
            this.editControl1.ContextChoiceBorderColor = System.Drawing.Color.FromArgb(((int)(((byte)(233)))), ((int)(((byte)(166)))), ((int)(((byte)(50)))));
            this.editControl1.ContextChoiceForeColor = System.Drawing.SystemColors.InfoText;
            this.editControl1.ContextPromptBackgroundBrush = new Syncfusion.Drawing.BrushInfo(System.Drawing.Color.FromArgb(((int)(((byte)(255)))), ((int)(((byte)(255)))), ((int)(((byte)(255))))));
            this.editControl1.ContextTooltipBackgroundBrush = new Syncfusion.Drawing.BrushInfo(System.Drawing.Color.FromArgb(((int)(((byte)(231)))), ((int)(((byte)(232)))), ((int)(((byte)(236))))));
            this.editControl1.Dock = System.Windows.Forms.DockStyle.Fill;
            this.editControl1.FilterAutoCompleteItems = false;
            this.editControl1.GraphicsTextRenderingHint = System.Drawing.Text.TextRenderingHint.ClearTypeGridFit;
            this.editControl1.IndicatorMarginBackColor = System.Drawing.Color.Empty;
            this.editControl1.LineNumbersColor = System.Drawing.Color.FromArgb(((int)(((byte)(0)))), ((int)(((byte)(128)))), ((int)(((byte)(128)))));
            this.editControl1.LineNumbersFont = new System.Drawing.Font("Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((byte)(204)));
            this.editControl1.Location = new System.Drawing.Point(10, 40);
            this.editControl1.Name = "editControl1";
            this.editControl1.RenderRightToLeft = false;
            this.editControl1.SaveOnClose = false;
            this.editControl1.ScrollPosition = new System.Drawing.Point(0, 0);
            this.editControl1.SelectionTextColor = System.Drawing.Color.FromArgb(((int)(((byte)(173)))), ((int)(((byte)(214)))), ((int)(((byte)(255)))));
            this.editControl1.ShowIndicatorMargin = false;
            this.editControl1.ShowSelectionMargin = false;
            this.editControl1.Size = new System.Drawing.Size(524, 407);
            this.editControl1.StatusBarSettings.CoordsPanel.Width = 150;
            this.editControl1.StatusBarSettings.EncodingPanel.Width = 100;
            this.editControl1.StatusBarSettings.FileNamePanel.Width = 100;
            this.editControl1.StatusBarSettings.InsertPanel.Width = 33;
            this.editControl1.StatusBarSettings.Offcie2007ColorScheme = Syncfusion.Windows.Forms.Office2007Theme.Blue;
            this.editControl1.StatusBarSettings.Offcie2010ColorScheme = Syncfusion.Windows.Forms.Office2010Theme.Blue;
            this.editControl1.StatusBarSettings.StatusPanel.Width = 70;
            this.editControl1.StatusBarSettings.TextPanel.Width = 80;
            this.editControl1.StatusBarSettings.VisualStyle = Syncfusion.Windows.Forms.Tools.Controls.StatusBar.VisualStyle.Default;
            this.editControl1.TabIndex = 0;
            this.editControl1.Text = "editControl1";
            this.editControl1.TransferFocusOnTab = true;
            this.editControl1.UseXPStyleBorder = true;
            this.editControl1.VisualColumn = 1;
            this.editControl1.VScrollMode = Syncfusion.Windows.Forms.Edit.ScrollMode.Immediate;
            this.editControl1.WordWrap = true;
            this.editControl1.ZoomFactor = 1F;
            this.editControl1.ContextChoiceBeforeOpen += new System.ComponentModel.CancelEventHandler(this.editControl1_ContextChoiceBeforeOpen);
            this.editControl1.ContextChoiceOpen += new Syncfusion.Windows.Forms.Edit.ContextChoiceEventHandler(this.editControl1_ContextChoiceOpen);
            this.editControl1.ContextPromptBeforeOpen += new System.ComponentModel.CancelEventHandler(this.editControl1_ContextChoiceBeforeOpen);          
            // 
            // mainFrameBarManager1
            // 

            // 
            // Form1
            // 
            this.AutoScaleBaseSize = new System.Drawing.Size(5, 15);
            this.ClientSize = new System.Drawing.Size(544, 457);
            this.Controls.Add(this.editControl1);
            this.Font = new System.Drawing.Font("Segoe UI", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
            this.Icon = ((System.Drawing.Icon)(resources.GetObject("$this.Icon")));
            this.MinimumSize = new System.Drawing.Size(556, 494);
            this.Name = "Form1";
            this.Padding = new System.Windows.Forms.Padding(10);
            this.StartPosition = System.Windows.Forms.FormStartPosition.CenterScreen;
            this.Text = "Advanced Intellisense";
            this.WindowState = System.Windows.Forms.FormWindowState.Maximized;
             
            ((System.ComponentModel.ISupportInitialize)(this.editControl1)).EndInit();

            this.ResumeLayout(false);

        }
        #endregion

        #region MAIN Method
        /// <summary>
        /// The main entry point for the application.
        /// </summary>
        [STAThread]
        static void Main()
        {
            Application.EnableVisualStyles();
            Application.Run(new Form1());
        }
        #endregion

        #region Class Helper Methods
        /// <summary>
        /// Looks for the configuration with given name in the specified container.
        /// </summary>
        /// <param name="container"></param>
        /// <param name="lexemStart"></param>
        /// <returns></returns>
        private ConfigLexem GetConfigLexem(IConfigLexem container, string lexemStart)
        {
            ConfigLexem lexem = null;

            // Searches configuration.
            foreach (IConfigLexem lexemTemp in container.SubLexems)
            {
                if (lexemTemp.IsEqualToBegin(lexemStart))
                {
                    lexem = lexemTemp as ConfigLexem;
                    break;
                }
            }

            // If it was not found, then create new one.
            if (lexem == null)
            {
                lexem = new ConfigLexem(lexemStart, "", FormatType.Text, false);
                container.SubLexems.Add(lexem);

                // ParentConfig will be updated in ResetCaches, but the newly created configuration cannot be used without ParentConfig being set.
                lexem.ParentConfig = container;
            }

            return lexem;
        }

        /// <summary>
        /// Looks for the first lexeme on the left with DropContextChoiceList set to true in config
        /// </summary>
        /// <returns></returns>
        private ILexem FindDropper()
        {
            ILexemLine line = editControl1.CurrentLineInstance;

            if (line == null) return null;

            ILexem lexem = line.FindLexemByColumn(editControl1.CurrentColumn);

            if (lexem == null)
            {
                // If we are in the virtual space in the line.
                if (editControl1.CurrentColumn > 1)
                    lexem = line.FindLexemByColumn(line.LineLength);

                if (lexem == null) return null;
            }

            // If current lexem is dropper itself
            if (lexem.Config.DropContextChoiceList)
                return lexem;

            int index = line.LineLexems.IndexOf(lexem);

            while (index > 0 && !lexem.Config.DropContextChoiceList)
                lexem = line.LineLexems[--index] as ILexem;

            if (index <= 0) lexem = null;

            return lexem;
        }

        /// <summary>
        /// Gets list of the items to be put to the context choice list.
        /// </summary>
        /// <returns></returns>
        private ICollection GetList()
        {
            // Gets neares context choice list "dropper"
            ILexem lexem = FindDropper();

            ArrayList list = new ArrayList();
            if (lexem == null) return list;

            // Adds to the resulting list all lexeme configuratios with "Method" and "Property" formats.
            foreach (IConfigLexem confLex in lexem.Config.SubLexems)
            {
                if (confLex.Format.Name == "Method")
                    list.Add(confLex);

                if (confLex.Format.Name == "Property")
                    list.Add(confLex);
            }

            return list;
        }

        #endregion

        #region Class Event Handlers
        /// <summary>
        /// Checks wether ContextChoice menu can be dropped.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void editControl1_ContextChoiceBeforeOpen(object sender, System.ComponentModel.CancelEventArgs e)
        {
            e.Cancel = (FindDropper() == null);
        }
        /// <summary>
        /// Fills ContextChoiceMenu according to the current context.
        /// </summary>
        /// <param name="form"></param>
        private void editControl1_ContextChoiceOpen(IContextChoiceController controller)
        {
            // Gets the list of Method and Property configurations for current context.
            ICollection list = GetList();

            int imageIndex = 0;
            // Adds configuration BeginBlocks and associated descriptions to the Context Choice List.
            foreach (IConfigLexem conf in list)
            {
                controller.Items.Add(conf.BeginBlock, (string)m_MethodComments[conf.ID], this.editControl1.ContextChoiceController.Images["Image" + imageIndex.ToString()]);
                imageIndex++;
            }
        }

        #endregion

    }
}
  
